@@ -25,7 +25,6 @@
 #include <linux/soc/qcom/mdt_loader.h>
 #include <linux/soc/qcom/smem.h>
 #include <linux/soc/qcom/smem_state.h>
-#include <linux/rpmsg/qcom_smd.h>
 
 #include "qcom_common.h"
 #include "remoteproc_internal.h"
@@ -468,7 +467,6 @@ static int wcnss_request_irq(struct qcom_wcnss *wcnss,
 			     irq_handler_t thread_fn)
 {
 	int ret;
-	int irq_number;
 
 	ret = platform_get_irq_byname(pdev, name);
 	if (ret < 0 && optional) {
@@ -479,19 +477,14 @@ static int wcnss_request_irq(struct qcom_wcnss *wcnss,
 		return ret;
 	}
 
-	irq_number = ret;
-
 	ret = devm_request_threaded_irq(&pdev->dev, ret,
 					NULL, thread_fn,
 					IRQF_TRIGGER_RISING | IRQF_ONESHOT,
 					"wcnss", wcnss);
-	if (ret) {
+	if (ret)
 		dev_err(&pdev->dev, "request %s IRQ failed\n", name);
-		return ret;
-	}
 
-	/* Return the IRQ number if the IRQ was successfully acquired */
-	return irq_number;
+	return ret;
 }
 
 static int wcnss_alloc_memory_region(struct qcom_wcnss *wcnss)
